import xml.etree.ElementTree as ET
import matplotlib.pyplot as plt
import numpy as np
import math
from pycbc import types, fft, waveform

def arrondir_maximisant(nombre):
    return 10**math.ceil(math.log10(nombre))

def get_nb_element_inf_seuil(liste, seuil):
    """Retourne les éléments de la liste qui sont inférieurs au seuil."""
    n = 0
    for element in liste:
        if element < seuil:
            n += 1
        else:
            # Arrêt de la boucle dès qu'on atteint un élément supérieur au seuil
            break
    return n

# Chemin du fichier XML
xml_path = "data/G463327/coinc.xml"
xml_path = "data/G408702/coinc.xml"

# Parsage du fichier XML
tree = ET.parse(xml_path)
root = tree.getroot()

# Initialiser des listes pour stocker les données de H1 et L1
x_values_h1, y_values_h1 = [], []
x_values_l1, y_values_l1 = [], []

# Parcourir les balises <LIGO_LW Name="REAL8FrequencySeries"> pour H1 et L1
for lw_element in root.findall(".//LIGO_LW[@Name='REAL8FrequencySeries']"):
    instrument_param = lw_element.find(".//Param[@Name='instrument:param']")
    if instrument_param is not None:
        instrument_name = instrument_param.text.strip()  # Assurez-vous de supprimer les espaces blancs autour du texte
        stream_element = lw_element.find(".//Stream[@Type='Local']")
        if stream_element is not None:
            data_str = stream_element.text.strip()
            data_list = [float(value) for value in data_str.split()]

            # Séparer les données en paires (x, y)
            x_values = data_list[::2]
            y_values = data_list[1::2]

            # Stocker les données en fonction de l'instrument (H1 ou L1)
            if "H1" in instrument_name and "G408702" not in xml_path:
                x_values_h1.extend(x_values)
                y_values_h1.extend(y_values)
            elif "L1" in instrument_name:
                x_values_l1.extend(x_values)
                y_values_l1.extend(y_values)

seuil_f_min = 10.0

n_x_h1 = get_nb_element_inf_seuil(x_values_h1, seuil_f_min)
n_x_l1 = get_nb_element_inf_seuil(x_values_l1, seuil_f_min)

if(n_x_h1 > 0):
    x_values_h1 = x_values_h1[n_x_h1 - 1:]
    y_values_h1 = y_values_h1[n_x_h1 - 1:]

if(n_x_l1 > 0):
    x_values_l1 = x_values_l1[n_x_l1 - 1:]
    y_values_l1 = y_values_l1[n_x_l1 - 1:]

# Calculer la racine carrée des valeurs de l'axe y
y_values_h1_sqrt = np.sqrt(y_values_h1)
y_values_l1_sqrt = np.sqrt(y_values_l1)

if "G408702" in xml_path:
    seuil_f_max = x_values_l1[-1]
    min_PSD = min(y_values_l1)
    min_PSD_sqrt = min(y_values_l1_sqrt)
    max_PSD = max(y_values_l1)
    max_PSD_sqrt = max(y_values_l1_sqrt)

else:
    seuil_f_max = max(x_values_l1[-1], x_values_h1[-1])
    min_PSD = min(min(y_values_h1), min(y_values_l1))
    min_PSD_sqrt = min(min(y_values_h1_sqrt), min(y_values_l1_sqrt))
    
    max_PSD = max(max(y_values_h1), max(y_values_l1))
    max_PSD_sqrt = max(max(y_values_h1_sqrt), max(y_values_l1_sqrt))


"""for mass in (1.4, 5):
    sptilde, sctilde = waveform. get_fd_waveform(approximant="IMRPhenomD", mass1=mass, mass2=mass, delta_f=1.0/4, f_lower=seuil_f_min)
    plt.plot(sctilde.sample_frequencies,abs(sctilde), linestyle='--', label="m1=m2=%s" % mass)
    max_PSD_sqrt = max(max_PSD_sqrt, max(abs(sctilde)))

mass1 = 114.96318
mass2 = 16.413901
spin1 = 0.115128
spin2 = 0.115128
distance = 2699.155338  # distance en mégaparsecs
sptilde, sctilde = waveform. get_fd_waveform(approximant="IMRPhenomD", mass1=mass1, mass2=mass2, spin1z=spin1, spin2z=spin2, delta_f=1.0/4, f_lower=seuil_f_min, distance=distance)
plt.plot(sctilde.sample_frequencies,abs(sctilde), linestyle='--', label=f"m1={mass1:.2f}, m2={mass2:.2f}, spin1z={spin1:.2f}, spin2z={spin2:.2f}, {distance:.2f}Mpc")
max_PSD_sqrt = max(max_PSD_sqrt, max(abs(sctilde)))"""

# Tracer les données avec matplotlib sans points et échelle logarithmique sur l'axe y
if "G408702" not in xml_path:
    plt.plot(x_values_h1, y_values_h1_sqrt, linestyle='-', marker='', color='r', label='H1', linewidth=0.75)  # Ajuster linewidth
plt.plot(x_values_l1, y_values_l1_sqrt, linestyle='-', marker='', color='b', label='L1', linewidth=0.75)  # Ajuster linewidth


# Ajouter des traits verticaux pour chaque multiple de puissance intermédiaire
seuil_f_max_arrondit = arrondir_maximisant(seuil_f_max)

power_of_ten = 10

while power_of_ten <= seuil_f_max_arrondit:
    plage = list(range(power_of_ten//10, power_of_ten, power_of_ten//10))[1:]
    for intermediate in plage:
        if intermediate < seuil_f_max:
            plt.axvline(intermediate, color='grey', linestyle='--', linewidth=0.5)
        else:
            break
    power_of_ten *= 10


# Définir les limites des axes x et y

plt.xlim(seuil_f_min, seuil_f_max)
plt.ylim(min_PSD_sqrt, max_PSD_sqrt)

plt.xlabel("Frequency (Hz)")
plt.ylabel("Spectral Density (strain/Hz)")
plt.title("Square Root of REAL8FrequencySeries for H1 and L1")
plt.xscale('log')
plt.yscale('log')  # Utilisez une échelle logarithmique sur l'axe y
plt.legend()  # Ajouter une légende pour distinguer H1 et L1

# Ajouter une grille
plt.grid(True)

# Enregistrer le graphique en tant que fichier PNG
plt.savefig("plot.png")
plt.close()

print("Graphique enregistré en tant que plot.png.")

